;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xe6c23dc0c2e1b8b0c17f28ee7e95d913 . "./code.cg2js/cg-acache.fasl")
(#x83b9e313a5d3edab97b53c4b358397cf . "./code.cg2js/cg-base.fasl")
(#x0cf02e789b1325705cad24b2eaf0cfe3 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xf5faef30546d7491c248f88dc2a08f0e . "./code.cg2js/cg-bitmap-pane.fasl")
(#xd50f35b45d5ab70e56687ef6d2afb6d2 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x2cd670832ad6601490a88f821c4ceed1 . "./code.cg2js/cg-button.fasl")
(#xd1664d64a879f82b9aa73cb11fc29283 . "./code.cg2js/cg-calendar.fasl")
(#xc65e1e7f337003332e66128d67067c82 . "./code.cg2js/cg-caret.fasl")
(#x7c6cf6a95a2cb91bedbc04a4d17c0563 . "./code.cg2js/cg-chart-or-plot.fasl")
(#xc271626edf562a5ce01ddc42446289bd . "./code.cg2js/cg-chart-widget.fasl")
(#xc9c6e49cd104a39d3998dd71bd4c661b . "./code.cg2js/cg-check-box.fasl")
(#x7b88b46f6f8a69512313a86e9236bcf7 . "./code.cg2js/cg-choice-list.fasl")
(#x2af393def4283ea1ea813bd201c39de6 . "./code.cg2js/cg-choose-printer.fasl")
(#x70175fce85cd3d56ab197dfad02e5f88 . "./code.cg2js/cg-class-grid.fasl")
(#x35dd2787a1bd67c9411c63923fb65f69 . "./code.cg2js/cg-class-slot-grid.fasl")
(#xb0c568529078de9c6d3fcdc3c24ad494 . "./code.cg2js/cg-class-support.fasl")
(#x441ba043221fb9dffda9c0d5cf9e3ec8 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xbe061a810bf3f048984283224d78f638 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x54c1b469b79fce349c362e8508816772 . "./code.cg2js/cg-clipboard.fasl")
(#xffa88bfb37b6fc29ac4a01270133e828 . "./code.cg2js/cg-color-dialog.fasl")
(#x9d10468e0062bf063f5f28c70d7fff58 . "./code.cg2js/cg-combo-box.fasl")
(#xdc175acd0fe366f8e7afcd7d5c4594d8 . "./code.cg2js/cg-common-control.fasl")
(#x4b789b0d0a5558a0a88b19fd2f153a02 . "./code.cg2js/cg-comtab.fasl")
(#x01e0b32d000f5ed6eda0558998a25c70 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x718befa719201e6572f5ce2f0ae5d073 . "./code.cg2js/cg-curve.fasl")
(#x941de16b93326d7e208e9c1fe48c7a8b . "./code.cg2js/cg-dde-utils.fasl")
(#x62ac57c7ed939db4959269c56a794124 . "./code.cg2js/cg-dev.fasl")
(#x75bc994844a6156c9ee7dfc015144f9c . "./code.cg2js/cg-dialog-item.fasl")
(#x52c9bfa207356d6488ff3fe94db1dbe4 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xcc1559237f42fa6d6dcf8954454a17e8 . "./code.cg2js/cg-directory-dialog.fasl")
(#x01cd1f52e46f74de8b0fab8b892a795e . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x6aa62cb0bdf3d91269140fe18bbe54ba . "./code.cg2js/cg-drag-and-drop.fasl")
(#x2ec6325f3f8d5b246cec8332436363b8 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x21275178f1478545a571a871657e22e4 . "./code.cg2js/cg-drawable.fasl")
(#xd1ff5d7f6ea85c1401ce4ac94acca2b5 . "./code.cg2js/cg-dropping-outline.fasl")
(#xb716cf5c17a1ec0134dc833d5b740b58 . "./code.cg2js/cg-edit-in-place.fasl")
(#x95976f450959edff754700b42a3806c4 . "./code.cg2js/cg-editable-text.fasl")
(#xdc0c77bfeee410ae36d78fc5a0ffe9bf . "./code.cg2js/cg-file-dialog.fasl")
(#x838bfdaafebaa6ad971007c2534061bd . "./code.cg2js/cg-fill-texture.fasl")
(#x9975505406dff23e6165c974735b1676 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x885e8c485614957ef683f7dda2c0a6e1 . "./code.cg2js/cg-font-dialog.fasl")
(#x618a50d18384b393e8d8dd669b66ed3a . "./code.cg2js/cg-gesture-emulation.fasl")
(#xd3e8159d7400900f823e290aa05c9681 . "./code.cg2js/cg-get-pixmap.fasl")
(#x7a5cc0a8e5b894bd86d01d11ac2aab3d . "./code.cg2js/cg-get-position.fasl")
(#x9755263e61e0f1554e6f213b59c6d8da . "./code.cg2js/cg-graphics-context.fasl")
(#xb27403f7f6810a0cc7be2dd4c79fca6d . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xa791bfecc3dd4bf3a3a47e0fcfb61df2 . "./code.cg2js/cg-grid-widget.fasl")
(#x26383d3242d0710a5350e6dcbb1a0d08 . "./code.cg2js/cg-group-box.fasl")
(#x567eef3e7dc3401624a853b43f3d07a3 . "./code.cg2js/cg-header-control.fasl")
(#x30bdea4a3649cefc4c4d723441147ef2 . "./code.cg2js/cg-hotspot.fasl")
(#x747ca70463d5bb3b52b2f063d8535866 . "./code.cg2js/cg-html-dialog.fasl")
(#xa761ab6afc22131af485dd34e626906b . "./code.cg2js/cg-html-widget.fasl")
(#xe6a642925f4d5a28e55aa1f64f8ac128 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xb50062ff4211093c034d841cc3a929aa . "./code.cg2js/cg-icon.fasl")
(#x2e9059134bef245c7c5313d870314f53 . "./code.cg2js/cg-ie.fasl")
(#x903df8c08807e3b9c5bfd938ae01646a . "./code.cg2js/cg-intersect-posbox.fasl")
(#xb78229da9905d36c261e32261916f957 . "./code.cg2js/cg-intersect.fasl")
(#xf84cb2617ff9072b34d88349f0afda57 . "./code.cg2js/cg-item-list.fasl")
(#x925b34ba33f1954e4e4ce36afed2bcd8 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xeba0e9695fd1ae91cd7e67f90a5d6d5f . "./code.cg2js/cg-lamp.fasl")
(#xdc3f254238c8bba761b2b5a938049840 . "./code.cg2js/cg-layout.fasl")
(#x6b3fc700cbed28377733967b97091454 . "./code.cg2js/cg-lettered-menu.fasl")
(#xf7a1892761b2d4b6e3536452e979680f . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x7b70be59169d4d26898d788482ea5567 . "./code.cg2js/cg-lisp-text.fasl")
(#xcc25493e54c7b2040c3de355ff71106b . "./code.cg2js/cg-lisp-widget.fasl")
(#x3d67aa917e5f71f1f634a36a6b9299ee . "./code.cg2js/cg-list-view.fasl")
(#x09bf1dd6e159b468056aa136369686bf . "./code.cg2js/cg-mci.fasl")
(#x9080479510b2c8e911253621da22b277 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x041d98a6ad2167ce41949f07d50eab7f . "./code.cg2js/cg-menu.fasl")
(#x2378a59c48f8108baceb38ac3ae795f9 . "./code.cg2js/cg-message-dialog.fasl")
(#xbd7949e8af8bb10a59ab1e26d3947473 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xd7eb10a5ede9f4dc8050bfe13fc33a01 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x229226e2b4d3a4eac2acdfc2d7939077 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x4fa87ee67447acc8dbd427f5cf26f77f . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xfd2bd92ef6aefeddf7e9169506bdaebe . "./code.cg2js/cg-multi-picture-button.fasl")
(#x4a11bf6a7db8c92962c2eddedfdecd2f . "./code.cg2js/cg-nodes.fasl")
(#xbabccfe3e62435e51e46bfae4fd99413 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x78604a7178e85f0b8f28138c97069600 . "./code.cg2js/cg-object-editor.fasl")
(#xdf0a7b98838315b48107f27648b8132f . "./code.cg2js/cg-ocx.fasl")
(#x09183363cb9b8c3ca289e794ea832640 . "./code.cg2js/cg-os-widget.fasl")
(#xfadf67588622280ae631be325700ddfa . "./code.cg2js/cg-os-window.fasl")
(#xd987f1b2b65320f9dd83d091c1f8a3c8 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xe8fda6928ea9cd9b414d1e5717a593f1 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xfb3c817ea59adc159f42eb3a05f77975 . "./code.cg2js/cg-outline.fasl")
(#xa23d884b6c90388b2cbdb82ff79f754a . "./code.cg2js/cg-palette.fasl")
(#xc94660fdbdf6f41e142286a269b68a81 . "./code.cg2js/cg-paren-matching.fasl")
(#xd07d4299c260c2df171fa8f5baba3784 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x39d985490aa97d0500e8be58721fdf7a . "./code.cg2js/cg-picture-widget.fasl")
(#x4530fd9d78bdceb3aa380454b0febd69 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xea4bd7abc83158dfde007a8b2dd68f66 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x4aeecaf547456bcceccdf164b0012b38 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x89ff22f2a8ea3521d19f3fdb31707953 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x207999cc2eeac7cbfa8ba4a4c77e7e39 . "./code.cg2js/cg-pixmap.fasl")
(#x87dd2e1c32ef9b53be01b77aeaa66c1c . "./code.cg2js/cg-player.fasl")
(#x60205c85831a07feb8be2da6706efb13 . "./code.cg2js/cg-plot-widget.fasl")
(#x40d25c2dd5edd9c64abf77b8585e58f0 . "./code.cg2js/cg-printing.fasl")
(#x653ab24411814128538776c15c8d0f34 . "./code.cg2js/cg-progress-indicator.fasl")
(#x91b861a1c27c4b40aa1629279f394688 . "./code.cg2js/cg-project-window.fasl")
(#x9ada49d8114e35273f88ba9e4ff41870 . "./code.cg2js/cg-property.fasl")
(#x83decb32a4285a1ab9df56ed73129aa9 . "./code.cg2js/cg-radio-button.fasl")
(#x9ced7cff0f484b5812bf521e225bceaf . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x0e7a5fde72c01bf36900bb50ae3df072 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#xcee964b0e9485dc0a774e902122a7722 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x5da3157146eb517a239d3b2d175046b7 . "./code.cg2js/cg-rich-edit.fasl")
(#x6827ccf768a08398eaf3fb4b71d45133 . "./code.cg2js/cg-sample-file-menu.fasl")
(#xe4c0f7a160feffdfe2734be97016c333 . "./code.cg2js/cg-scaling-stream.fasl")
(#x81ff466cfd1f15f9c2a8643586002c57 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x95310eac68e46f0f0decb311f677d688 . "./code.cg2js/cg-scroll-bar.fasl")
(#x0500af6e47051f554c806436305a9827 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x4033829e39cf4082c3ce75553d3922c1 . "./code.cg2js/cg-selected-object.fasl")
(#x98a443a8a2bb69e04a998fbf8eefc320 . "./code.cg2js/cg-shortcut-menu.fasl")
(#xaa43f86d430fb6b70b920b1f9296c865 . "./code.cg2js/cg-split-bar.fasl")
(#x697cfe48bc43f9feca9f9e6dbfa5b13a . "./code.cg2js/cg-static-text.fasl")
(#xc9b3b9eb56e3fcadef8786512fe8e0d8 . "./code.cg2js/cg-status-bar.fasl")
(#x7694a5583cbe33662daf8aab8c24afde . "./code.cg2js/cg-string-dialog.fasl")
(#xc7495d936b3bb2a4ec57744cf639d556 . "./code.cg2js/cg-tab-control.fasl")
(#x3d5106b832cbdf0dba51321b209158c1 . "./code.cg2js/cg-template-string.fasl")
(#x5f0b42ecb42c1e92e0c2b07f915fe734 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xeb18bb90e15fd0eca71c7269a33268eb . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xdaccc536d12063696a5f3062fb048af6 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x1a1af0d2a7da636e664a1fcec33b5703 . "./code.cg2js/cg-text-or-combo.fasl")
(#x3c6680e1af7033d4372a0271103111fa . "./code.cg2js/cg-text-widget.fasl")
(#xbd847ec5c79b672b303dafa295276778 . "./code.cg2js/cg-timer.fasl")
(#x2acd4130ef252cf36b69031fd39639b4 . "./code.cg2js/cg-toggling-widget.fasl")
(#x21a6d04e348001e8af22c751a37dd141 . "./code.cg2js/cg-toolbar.fasl")
(#x391fbc6ab266eec1f16fa963e402731b . "./code.cg2js/cg-tooltip.fasl")
(#x027bfc97b84ab167324be3f5dc42019b . "./code.cg2js/cg-touch.fasl")
(#x2f9ab734b5715f64279928368ee6fa0b . "./code.cg2js/cg-trackbar.fasl")
(#xe4a17de8b4b0e40c51cb878dac7e1463 . "./code.cg2js/cg-tray.fasl")
(#x97b7810d94c87d373964450b74d2bd12 . "./code.cg2js/cg-up-down-control.fasl")
(#xd27edf761615ddd1507b4a3e72c51270 . "./code.cg2js/cg-utility-dialog.fasl")
(#x909c2300bf441f9ec7631a9c4d451c15 . "./code.cg2js/cg-web-browser-dde.fasl")
(#xdac522c7b03a85321c220bfa36aa2e2e . "./code.cg2js/cg-web-browser.fasl")
(#x14bce8b4cd95ea84fe989e61c0a2adc1 . "./code.cg2js/cg-wrap-string.fasl")
(#xd7975ab3f821b01a2aa5eacba88b7726 . "./code.cg2js/cg-yes-no-list.fasl")
(#xe873b1d4fd16cb48fefdb23c554b8559 . "./code.cg2js/cg.fasl")
(#xc8dcee5e7d00db08b825ffa9ae9c2729 . "./code.cg2js/cg-yes-no-string.fasl")
(#xeb1123980e205b8f96ff1f1b109d0769 . "./code.idejs/ide-acache.fasl")
(#x9fa0309821399a2bddc3d04e73b78eef . "./code.idejs/ide-apropos.fasl")
(#x9c331fde6846aa36b29a17d04cb6a9c9 . "./code.idejs/ide-base.fasl")
(#xfb60607f15e7321062ba1ff91a883a81 . "./code.idejs/ide-class-browser.fasl")
(#x726772de526d2b70b45c6b0b968f6658 . "./code.idejs/ide-class-grapher.fasl")
(#x811aeaaac6dda30f0693944c5136aaa7 . "./code.idejs/ide-clipboard.fasl")
(#xaacf8951c6b00c13e2eb59f956b20c0f . "./code.idejs/ide-dev.fasl")
(#x488912012289ee2b43a77c2fab92721d . "./code.idejs/ide-editor.fasl")
(#x2c99d3ae66f8fddc43cff6253adcfe0f . "./code.idejs/ide-find-definitions.fasl")
(#xb09c7cd06db548cab5a826d45db9755b . "./code.idejs/ide-find-in-files.fasl")
(#x15c16778d01c9f094b889c9c5a31a3e9 . "./code.idejs/ide-form.fasl")
(#x50dd8e0203a73755d4a386ebfec1c011 . "./code.idejs/ide-grapher.fasl")
(#x23986af1da861e952d917f718ac5d3d2 . "./code.idejs/ide-inspector.fasl")
(#x379015573deb5b5bca5f44785af52e08 . "./code.idejs/ide-intro.fasl")
(#x31e441842bb9fbac5050989b12cdb6e6 . "./code.idejs/ide-keyshort.fasl")
(#xada6a8acc01153f3b53b0b32d23bcb84 . "./code.idejs/ide-navigator.fasl")
(#x96b8653a5920af75393854ece0ebaf4b . "./code.idejs/ide-package-list.fasl")
(#xcde09f66e2b17aebd2ebf78f241c4b9b . "./code.idejs/ide-patch-dialog.fasl")
(#x936df827f77422988e3c164c0544a261 . "./code.idejs/ide-process-browser.fasl")
(#xd847a803a54bbd7dff4652deec4c7aea . "./code.idejs/ide-project.fasl")
(#xc1b7482487ab0947930fc5018ce53de2 . "./code.idejs/ide-trace.fasl")
(#xfc53bcc9f42a3c8726d713128d56aeaa . "./code.idejs/ide-window-list.fasl")
(#x8422667a04cefad35373aaaceacee267 . "./code.idejs/ide.fasl")
(#xde049c9404b4ba293f0cbc6a4817dc96 . "./cgjs.html")
(#x55fea08d92e113f5a8381c271febbc9e . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (format t "Incorrect checksum on file: ~a" (cdr checksum))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
